import os
import datasets
import numpy as np

from fewshot_gym_dataset import FewshotGymDataset, FewshotGymClassificationDataset


class Glue_SST2(FewshotGymClassificationDataset):
    def __init__(self):
        self.hf_identifier = "glue-sst2"

        self.task_type = "classification"

        # for classification tasks, specify the meaning of each label
        self.label = {
            0: "negative",
            1: "positive",
        }

    def map_hf_dataset_to_list(self, hf_dataset, split_name):
        lines = []
        for datapoint in hf_dataset[split_name]:
            # line[0]: input; line[1]: output
            lines.append(
                ("sentence: " + datapoint["sentence"], self.label[datapoint["label"]])
            )
        return lines

    def load_dataset(self):
        return datasets.load_dataset("glue", "sst2")


def main():
    dataset = Glue_SST2()

    for seed in [100, 13, 21, 42, 87]:
        train, dev, test = dataset.generate_k_shot_data(seed=seed, path="../data/")


if __name__ == "__main__":
    main()
